### Name: fkf
### Title: Fast Kalman filter
### Aliases: fkf
### Keywords: algebra models multivariate

### ** Examples

## This example shows how to fit an AR(2) model using the Kalman filter.

n <- 200

## Set the AR parameters
ar1 <- -0.24
ar2 <- 0.6
sigma <- .4

## Sample from an AR(2) process
a <- arima.sim(model = list(ar = c(ar1, ar2)),
               n = n, innov = rnorm(n) * sigma)

## Create a state space representation out of the three AR parameters
ar2.to.state.space <- function(ar1, ar2, sigma){
    Tt <- matrix(c(ar1, 1, ar2, 0), ncol = 2)
    Zt <- matrix(c(1, 0), ncol = 2)
    ct <- matrix(0.0)
    dt <- matrix(0.0, nrow = 2)
    GGt <- matrix(0.0)
    HHt <- matrix(c(sigma^2, 0, 0, 0), ncol = 2)
    a0 <- as.numeric(c(0, 0))
    P0 <- matrix(c(100,0,0,100), ncol = 2)
    return(list(a0 = a0, P0 = P0, ct = ct, dt = dt,
                Zt = Zt, Tt = Tt, GGt = GGt, HHt = HHt))
}

## The objective function passed to 'optim'
objective <- function(theta, yt)
{
    sp <- ar2.to.state.space(theta["ar1"], theta["ar2"], theta["sigma"])

    ans <- fkf(a0 = sp$a0, P0 = sp$P0,
               dt = sp$dt, ct = sp$ct, Tt = sp$Tt, Zt = sp$Zt,
               HHt = sp$HHt, GGt = sp$GGt, yt = yt, check.input = FALSE)

    return(-ans$logLik)
}

theta <- c(ar1 = 0, ar2 = 0, sigma = 1)
fit <- optim(theta, objective, yt = rbind(a))
fit

## Filter the series with estimated parameter values
sp <- ar2.to.state.space(fit$par["ar1"], fit$par["ar2"], fit$par["sigma"])

ans <- fkf(a0 = sp$a0, P0 = sp$P0,
           dt = sp$dt, ct = sp$ct, Tt = sp$Tt, Zt = sp$Zt,
           HHt = sp$HHt, GGt = sp$GGt, yt = rbind(a))

## Compare the prediction with the realization
plot(ans, at.idx=1, att.idx = NA, CI = NA)
lines(a, lty = "dotted")

## Compare the filtered series with the realization
plot(ans, at.idx=NA, att.idx = 1, CI = NA)
lines(a, lty = "dotted")

## Check whether the residuals are Gaussian
plot(ans, type = "resid.qq")

## Check for linear serial dependence through 'acf'
plot(ans, type = "acf")




